@@ -5,6 +5,32 @@ Constants for csle-common
 import re
 
 
+class GENERAL:
+    """
+    General constants
+    """
+    THREAD_PROPERTY = "thread"
+    PORT_PROPERTY = "port"
+    IP_PROPERTY = "ip"
+    EMULATION_PROPERTY = "emulation"
+    EXECUTION_ID_PROPERTY = "execution_id"
+
+
+class GRPC_SERVERS:
+    """
+    Constants related to grpc servers
+    """
+    CLUSTER_MANAGER_PORT = 50041
+    RYU_MANAGER_PORT = 50042
+    TRAFFIC_MANAGER_PORT = 50043
+    CLIENT_MANAGER_PORT = 50044
+    ELK_MANAGER_PORT = 50045
+    DOCKER_STATS_MANAGER_PORT = 50046
+    OSSEC_IDS_MANAGER_PORT = 50047
+    SNORT_IDS_MANAGER_PORT = 50048
+    HOST_MANAGER_PORT = 50049
+
+
 class CONFIG_FILE:
     """
     Constants related to the config file
@@ -147,6 +173,13 @@ class NETWORKING:
     ETH1 = "eth1"
     ETH2 = "eth2"
     ETH3 = "eth3"
+    ETH4 = "eth4"
+    ETH5 = "eth5"
+    ETH6 = "eth6"
+    ETH7 = "eth7"
+    ETH8 = "eth8"
+    ETH9 = "eth9"
+    ETH10 = "eth10"
 
 
 class SERVICES:
@@ -492,6 +525,7 @@ class SSH:
     SERVICE_NAME = "ssh"
     DEFAULT_PORT = 22
     DIRECT_CHANNEL = "direct-tcpip"
+    MAX_FILE_READ_BYTES = 5
 
 
 class TELNET:
@@ -854,11 +888,18 @@ class COMMANDS:
     START_DOCKER_STATS_MANAGER = "nohup csle statsmanager {} " \
                                  f"& > {DOCKER_STATS_MANAGER_OUTFILE} " \
                                  f"&& echo $! > {DOCKER_STATS_MANAGER_PIDFILE}"
+    CLUSTER_MANAGER_PIDFILE = "/var/log/csle/clustermanager.pid"
+    CLUSTER_MANAGER_OUTFILE = "/var/log/csle/clustermanager.out"
+    START_CLUSTER_MANAGER = "nohup csle clustermanager {} " \
+                            f"& > {CLUSTER_MANAGER_OUTFILE} " \
+                            f"&& echo $! > {CLUSTER_MANAGER_PIDFILE}"
     SEARCH_DOCKER_STATS_MANAGER = "statsmanager"
     SEARCH_PROMETHEUS = "prometheus"
     PROMETHEUS_PID_FILE = "/var/log/csle/prometheus.pid"
     PROMETHEUS_LOG_FILE = "/var/log/csle/prometheus.log"
     PROMETHEUS_CONFIG_FILE = f"${CONFIG_FILE.CSLE_HOME_ENV_PARAM}/management-system/prometheus/prometheus.yml"
+    POSTGRESQL_LOG_DIR = "/var/log/postgresql/"
+    NGINX_LOG_DIR = "/var/log/nginx/"
     PROMETHEUS_PORT = 9090
     START_PROMETHEUS = f"nohup prometheus --config.file={PROMETHEUS_CONFIG_FILE} " \
                        "--storage.tsdb.retention.size=10GB " \
@@ -869,14 +910,16 @@ class COMMANDS:
     NODE_EXPORTER_PORT = 9100
     GRAFANA_PORT = 3000
     MANAGEMENT_SYSTEM_PORT = 7777
+    FLASK_PORT = 7777
     START_GRAFANA = f"docker run -d -p {GRAFANA_PORT}:{GRAFANA_PORT} --name grafana grafana/grafana"
     CADVISOR_PORT = 8080
     PGADMIN_PORT = 7778
+    DOCKER_ENGINE_PORT = 2375
     START_CADVISOR = "docker run  -dt --volume=/:/rootfs:ro   --volume=/var/run:/var/run:ro   " \
                      "--volume=/sys:/sys:ro   " \
                      "--volume=/var/lib/docker/:/var/lib/docker:ro   --volume=/dev/disk/:/dev/disk:ro   " \
                      f"--publish={CADVISOR_PORT}:{CADVISOR_PORT}  --name=cadvisor  " \
-                     "google/cadvisor:latest"
+                     "gcr.io/cadvisor/cadvisor"
     PGADMIN_USERNAME = "csle@csle.com"
     PGADMIN_PW = "cslePassword"
     GRAFANA_USERNAME = "admin"
@@ -887,14 +930,18 @@ class COMMANDS:
     CADVISOR_LOGS = "docker logs cadvisor"
     PGADMIN_LOGS = "docker logs pgadmin"
     GRAFANA_LOGS = "docker logs grafana"
+    DOCKER_ENGINE_LOGS = "sudo /usr/bin/journalctl -u docker.service -n 100 --no-pager -e"
+    DOCKER_ENGINE_LOGS_ALTERNATIVE = "sudo /bin/journalctl -u docker.service -n 100 --no-pager -e"
     NODE_EXPORTER_PID_FILE = "/var/log/csle/node_exporter.pid"
-    MANAGEMENT_SYSTEM_PID_FILE = "/var/log/csle/management_system.pid"
+    CSLE_MGMT_WEBAPP_PID_FILE = "/var/log/csle/csle_mgmt_webapp.pid"
     NODE_EXPORTER_LOG_FILE = "/var/log/csle/node_exporter.log"
+    FLASK_LOG_FILE = "/var/log/csle/flask.log"
     START_NODE_EXPORTER = f"nohup node_exporter & > {NODE_EXPORTER_LOG_FILE} && echo $! " \
                           f"> {NODE_EXPORTER_PID_FILE}"
-    BUILD_MONITOR = f"cd ${CONFIG_FILE.CSLE_HOME_ENV_PARAM}/management-system/csle-mgmt-webapp && npm run build"
-    START_MONITOR = f"nohup python ${CONFIG_FILE.CSLE_HOME_ENV_PARAM}/management-system/csle-mgmt-webapp" \
-                    "/server/server.py &"
+    BUILD_CSLE_MGMT_WEBAPP = f"cd ${CONFIG_FILE.CSLE_HOME_ENV_PARAM}/management-system/csle-mgmt-webapp && npm run " \
+                             f"build"
+    START_CSLE_MGMT_WEBAPP = f"nohup python ${CONFIG_FILE.CSLE_HOME_ENV_PARAM}/management-system/csle-mgmt-webapp" \
+                             f"/server/server.py & > {FLASK_LOG_FILE}"
     GET_LATEST_PID = "$!"
     SAVE_PID = "echo {} > {}"
     KILL_PROCESS = "kill -9 {}"
@@ -956,6 +1003,7 @@ class FILE_PATTERNS:
     XML_FILE_SUFFIX = ".xm"
     CSV_SUFFIX = ".csv"
     LOG_SUFFIX = ".log"
+    GZ_SUFFIX = ".gz"
 
 
 class NIKTO:
@@ -1383,6 +1431,7 @@ class DOCKER:
     CREATE_USERS_SCRIPT = "./create_users.py"
     CREATE_TRAFFIC_GENERATORS_SCRIPT = "./create_traffic_generators.py"
     LIST_NETWORKS_CMD = "docker network ls"
+    INSPECT_DOCKER_GWBRIDGE = "docker network inspect docker_gwbridge"
     NETWORK_CONNECT = "docker network connect"
     LIST_RUNNING_CONTAINERS_CMD = "docker ps -q"
     INSPECT_CONTAINER_CONFIG_CMD = "docker inspect"
@@ -1400,6 +1449,8 @@ class DOCKER:
     IP_PREFIX_LEN_INFO = "IPPrefixLen"
     HOSTNAME_INFO = "Hostname"
     CONTAINERS_DIR = "containers"
+    CONTAINERS_KEY = "Containers"
+    IPV4_KEY = "IPv4Address"
     CONTAINER_MAKEFILE_TEMPLATE_NAME = "Container_Makefile_template"
     CONTAINER_MAKEFILE_TEMPLATE_DIR_RELATIVE = "/../../../common/"
     MAKEFILE_TEMPLATE = "Makefile_template"
@@ -1416,6 +1467,9 @@ class DOCKER:
         "docker container start $(PROJECT)-$(CONTAINER)$(SUFFIX)-level$(LEVEL)\n\nstop:\n\t" \
         "-docker stop $(PROJECT)-$(CONTAINER)$(SUFFIX)-level$(LEVEL)\n\nclean: stop\n\t" \
         "-docker rm $(PROJECT)-$(CONTAINER)$(SUFFIX)-level$(LEVEL)"
+    BRIDGE_NETWORK_DRIVER = "bridge"
+    OVERLAY_NETWORK_DRIVER = "overlay"
+    ATTACHABLE_NETWORK_FLAG = "--attachable"
 
 
 class CSLE:
