@@ -1,3 +1,4 @@
+import logging
 from typing import List, Tuple
 import subprocess
 import time
@@ -5,7 +6,6 @@ import docker
 import re
 import os
 import grpc
-import socket
 import csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc
 import csle_collector.docker_stats_manager.docker_stats_manager_pb2
 import csle_collector.docker_stats_manager.query_docker_stats_manager
@@ -14,15 +14,15 @@ from csle_common.util.docker_util import DockerUtil
 from csle_common.dao.emulation_config.emulation_env_config import EmulationEnvConfig
 from csle_common.dao.emulation_config.containers_config import ContainersConfig
 from csle_common.dao.emulation_config.container_network import ContainerNetwork
-from csle_common.dao.emulation_config.kafka_config import KafkaConfig
 from csle_common.dao.emulation_config.docker_stats_manager_config import DockerStatsManagerConfig
-from csle_common.dao.emulation_config.ovs_config import OVSConfig
 import csle_common.constants.constants as constants
 from csle_common.logging.log import Logger
 from csle_common.dao.emulation_config.emulation_execution import EmulationExecution
 from csle_common.dao.emulation_config.docker_stats_managers_info import DockerStatsManagersInfo
 from csle_common.dao.emulation_config.node_container_config import NodeContainerConfig
 from csle_common.controllers.management_system_controller import ManagementSystemController
+from csle_common.dao.emulation_config.config import Config
+from csle_common.util.general_util import GeneralUtil
 
 
 class ContainerController:
@@ -352,29 +352,36 @@ class ContainerController:
         return networks
 
     @staticmethod
-    def create_networks(containers_config: ContainersConfig) -> None:
+    def create_networks(containers_config: ContainersConfig, logger: logging.Logger) -> None:
         """
         Creates docker networks for a given containers configuration
 
+        :param logger: the logger to use for logging
         :param containers_config: the containers configuration
         :return: None
         """
         for c in containers_config.containers:
             for ip_net in c.ips_and_networks:
-                networks = ContainerController.get_network_references()
-                networks = list(map(lambda x: x.name, networks))
+                existing_networks = ContainerController.get_network_references()
+                existing_networks = list(map(lambda x: x.name, existing_networks))
                 ip, net = ip_net
-                ContainerController.create_network_from_dto(network_dto=net, existing_network_names=networks)
+                ContainerController.create_network_from_dto(network_dto=net, existing_network_names=existing_networks,
+                                                            logger=logger)
 
     @staticmethod
-    def connect_containers_to_networks(emulation_env_config: EmulationEnvConfig) -> None:
+    def connect_containers_to_networks(emulation_env_config: EmulationEnvConfig, physical_server_ip: str,
+                                       logger: logging.Logger) -> None:
         """
         Connects running containers to networks
 
         :param emulation_env_config: the emulation config
+        :param physical_server_ip: the ip of the physical server where the operation is executed
+        :param logger: the logger to use for logging
         :return: None
         """
         for c in emulation_env_config.containers_config.containers:
+            if c.physical_host_ip != physical_server_ip:
+                continue
             container_name = c.get_full_name()
             # Disconnect from none
             cmd = f"docker network disconnect none {container_name}"
@@ -387,22 +394,35 @@ class ContainerController:
                 ip, net = ip_net
                 cmd = f"{constants.DOCKER.NETWORK_CONNECT} --ip {ip} {net.name} " \
                       f"{container_name}"
-                Logger.__call__().get_logger().info(f"Connecting container:{container_name} to network:{net.name} "
-                                                    f"with ip: {ip}")
+                logger.info(f"Connecting container:{container_name} to network:{net.name} with ip: {ip}, cmd: {cmd}")
                 subprocess.Popen(cmd, stdout=subprocess.DEVNULL, shell=True)
 
-        ContainerController.connect_container_to_network(container=emulation_env_config.kafka_config.container)
-        ContainerController.connect_container_to_network(container=emulation_env_config.elk_config.container)
-        if emulation_env_config.sdn_controller_config is not None:
+                if c.docker_gw_bridge_ip == "" or c.docker_gw_bridge_ip is None:
+                    # Wait to make sure docker networks are updated
+                    time.sleep(2)
+                    # Extract the docker bridge gateway IP
+                    container_id = DockerUtil.get_container_hex_id(name=container_name)
+                    docker_gw_bridge_ip = DockerUtil.get_docker_gw_bridge_ip(container_id=container_id)
+                    c.docker_gw_bridge_ip = docker_gw_bridge_ip
+
+        if emulation_env_config.kafka_config.container.physical_host_ip == physical_server_ip:
+            ContainerController.connect_container_to_network(container=emulation_env_config.kafka_config.container,
+                                                             logger=logger)
+        if emulation_env_config.elk_config.container.physical_host_ip == physical_server_ip:
+            ContainerController.connect_container_to_network(container=emulation_env_config.elk_config.container,
+                                                             logger=logger)
+        if emulation_env_config.sdn_controller_config is not None and \
+                emulation_env_config.sdn_controller_config.container.physical_host_ip == physical_server_ip:
             ContainerController.connect_container_to_network(
-                container=emulation_env_config.sdn_controller_config.container)
+                container=emulation_env_config.sdn_controller_config.container, logger=logger)
 
     @staticmethod
-    def connect_container_to_network(container: NodeContainerConfig) -> None:
+    def connect_container_to_network(container: NodeContainerConfig, logger: logging.Logger) -> None:
         """
         Connect a running container to networks
 
         :param container: the container to connect
+        :param logger: the logger to use for logging
         :return: None
         """
         container_name = container.get_full_name()
@@ -417,16 +437,25 @@ class ContainerController:
             ip, net = ip_net
             cmd = f"{constants.DOCKER.NETWORK_CONNECT} --ip {ip} {net.name} " \
                   f"{container_name}"
-            Logger.__call__().get_logger().info(f"Connecting container:{container_name} to network:{net.name} "
-                                                f"with ip: {ip}")
+            logger.info(f"Connecting container:{container_name} to network:{net.name} with ip: {ip}, cmd: {cmd}")
             subprocess.Popen(cmd, stdout=subprocess.DEVNULL, shell=True)
+            if container.docker_gw_bridge_ip == "" or container.docker_gw_bridge_ip is None:
+                # Wait to make sure docker networks are updated
+                time.sleep(2)
+                # Extract the docker bridge gateway IP
+                container_id = DockerUtil.get_container_hex_id(name=container_name)
+                docker_gw_bridge_ip = DockerUtil.get_docker_gw_bridge_ip(container_id=container_id)
+                container.docker_gw_bridge_ip = docker_gw_bridge_ip
 
     @staticmethod
-    def start_docker_stats_thread(execution: EmulationExecution) -> None:
+    def start_docker_stats_thread(execution: EmulationExecution, physical_server_ip: str,
+                                  logger: logging.Logger) -> None:
         """
         Sends a request to the docker stats manager on the docker host for starting a docker stats monitor thread
 
         :param execution: the emulation execution
+        :param physical_server_ip: the ip of the physical server
+        :param logger: the logger to use for logging
         :return: None
         """
         if not ManagementSystemController.is_statsmanager_running():
@@ -437,9 +466,8 @@ class ContainerController:
                 max_workers=execution.emulation_env_config.docker_stats_manager_config.docker_stats_manager_max_workers
             )
             time.sleep(5)
-        hostname = socket.gethostname()
-        ip = socket.gethostbyname(hostname)
-        Logger.__call__().get_logger().info(
+        ip = physical_server_ip
+        logger.info(
             f"connecting to: {ip}:"
             f"{execution.emulation_env_config.docker_stats_manager_config.docker_stats_manager_port}")
         with grpc.insecure_channel(
@@ -448,26 +476,28 @@ class ContainerController:
             stub = csle_collector.docker_stats_manager.docker_stats_manager_pb2_grpc.DockerStatsManagerStub(channel)
             container_ip_dtos = []
             for c in execution.emulation_env_config.containers_config.containers:
-                name = c.get_full_name()
-                ip = c.get_ips()[0]
-                container_ip_dtos.append(csle_collector.docker_stats_manager.docker_stats_manager_pb2.ContainerIp(
-                    ip=ip, container=name))
-            Logger.__call__().get_logger().info("connected")
+                if c.physical_host_ip == physical_server_ip:
+                    name = c.get_full_name()
+                    ip = c.get_ips()[0]
+                    container_ip_dtos.append(csle_collector.docker_stats_manager.docker_stats_manager_pb2.ContainerIp(
+                        ip=ip, container=name))
+            logger.info("connected")
 
             csle_collector.docker_stats_manager.query_docker_stats_manager.start_docker_stats_monitor(
                 stub=stub, emulation=execution.emulation_name,
-                kafka_ip=execution.emulation_env_config.kafka_config.container.get_ips()[0],
+                kafka_ip=execution.emulation_env_config.kafka_config.container.docker_gw_bridge_ip,
                 stats_queue_maxsize=1000,
                 time_step_len_seconds=execution.emulation_env_config.docker_stats_manager_config.time_step_len_seconds,
-                kafka_port=execution.emulation_env_config.kafka_config.kafka_port,
+                kafka_port=execution.emulation_env_config.kafka_config.kafka_port_external,
                 containers=container_ip_dtos, execution_first_ip_octet=execution.ip_first_octet)
 
     @staticmethod
-    def stop_docker_stats_thread(execution: EmulationExecution) -> None:
+    def stop_docker_stats_thread(execution: EmulationExecution, physical_server_ip: str) -> None:
         """
         Sends a request to the docker stats manager on the docker host for stopping a docker stats monitor thread
 
         :param execution: the execution of the emulation for which the monitor should be stopped
+        :param physical_server_ip: the ip of the physical server
         :return: None
         """
         if not ManagementSystemController.is_statsmanager_running():
@@ -478,8 +508,7 @@ class ContainerController:
                 max_workers=execution.emulation_env_config.docker_stats_manager_config.docker_stats_manager_max_workers
             )
             time.sleep(5)
-        hostname = socket.gethostname()
-        ip = socket.gethostbyname(hostname)
+        ip = physical_server_ip
         with grpc.insecure_channel(
                 f'{ip}:'
                 f'{execution.emulation_env_config.docker_stats_manager_config.docker_stats_manager_port}') \
@@ -497,8 +526,7 @@ class ContainerController:
         :param docker_stats_manager_config: the docker stats manager configuration
         :return: None
         """
-        hostname = socket.gethostname()
-        ip = socket.gethostbyname(hostname)
+        ip = GeneralUtil.get_host_ip()
         docker_stats_monitor_dto = ContainerController.get_docker_stats_manager_status_by_ip_and_port(
             ip=ip, port=docker_stats_manager_config.docker_stats_manager_port)
         return docker_stats_monitor_dto
@@ -524,52 +552,33 @@ class ContainerController:
             return None
 
     @staticmethod
-    def connect_containers_to_management_network(containers_config: ContainersConfig, kafka_config: KafkaConfig,
-                                                 ovs_config: OVSConfig) -> None:
-        """
-        Connects running containers to the management network
-
-        :param containers_config: the containers configuration
-        :param kafka_config: the configuration of the kafka network
-        :param ovs_config: the OVS config
-        :return: None
-        """
-        kafka_server_ip, kafka_network = kafka_config.container.ips_and_networks[0]
-        kafka_network_prefix = ".".join(kafka_server_ip.split(".")[0:-1])
-        for c in containers_config.containers:
-            ovs = False
-            for ovs_image in constants.CONTAINER_IMAGES.OVS_IMAGES:
-                if ovs_image in c.name:
-                    ovs = True
-            if not ovs:
-                container_name = c.get_full_name()
-
-                ip_suffix = c.ips_and_networks[0][0].split(".")[-1]
-                c_ip = kafka_network_prefix + "." + ip_suffix
-                cmd = f"{constants.DOCKER.NETWORK_CONNECT} --ip {c_ip} {kafka_network.name} " \
-                      f"{container_name}"
-                Logger.__call__().get_logger().info(
-                    f"Connecting container:{container_name} to network:{kafka_network.name} with ip: {c_ip}")
-                subprocess.Popen(cmd, stdout=subprocess.DEVNULL, shell=True)
-
-    @staticmethod
-    def create_network_from_dto(network_dto: ContainerNetwork, existing_network_names=None) -> None:
+    def create_network_from_dto(network_dto: ContainerNetwork, logger: logging.Logger,
+                                existing_network_names=None) -> None:
         """
         Creates a network from a given DTO representing the network
 
+        :param network_dto: details of the network to create
         :param existing_network_names: list of network names, if not None, check if network exists befeore creating
+        :param logger: the logger to use for logging
         :return: None
         """
+        config = Config.get_current_config()
+        driver = constants.DOCKER.BRIDGE_NETWORK_DRIVER
+        if len(config.cluster_config.cluster_nodes) > 0:
+            driver = constants.DOCKER.OVERLAY_NETWORK_DRIVER
         ContainerController.create_network(name=network_dto.name, subnetmask=network_dto.subnet_mask,
-                                           existing_network_names=existing_network_names)
+                                           existing_network_names=existing_network_names, driver=driver,
+                                           logger=logger)
 
     @staticmethod
-    def create_network(name: str, subnetmask: str, driver: str = "bridge", existing_network_names: List = None) -> None:
+    def create_network(name: str, subnetmask: str, logger: logging.Logger,
+                       driver: str = "bridge", existing_network_names: List = None) -> None:
         """
         Creates a network
 
         :param name: the name of the network to create
         :param subnetmask: the subnetmask of the network to create
+        :param logger: the logger to use for logging
         :param driver: the driver of the network to create
         :param existing_network_names: list of network names, if not None, check if network exists befeore creating
         :return: None
@@ -585,68 +594,77 @@ class ContainerController:
         if existing_network_names is not None:
             network_names = existing_network_names
         if name not in network_names:
-            Logger.__call__().get_logger().info(f"Creating network: {name}, subnetmask: {subnetmask}")
+            logger.info(f"Creating network: {name}, subnetmask: {subnetmask}, driver: {driver}")
             client_1.networks.create(
                 name,
                 driver=driver,
-                ipam=ipam_config
+                ipam=ipam_config,
+                attachable=True
             )
 
     @staticmethod
-    def remove_network(name: str) -> None:
+    def remove_network(name: str, logger: logging.Logger) -> None:
         """
         Removes a network
 
         :param name: the name of the network to remove
+        :param logger: the logger to use for logging
         :return: None
         """
         client_1 = docker.from_env()
         networks = client_1.networks.list()
         for net in networks:
             if net.name == name:
-                Logger.__call__().get_logger().info(f"Removing network: {net.name}")
+                logger.info(f"Removing network: {net.name}")
                 try:
                     net.remove()
                 except Exception:
                     pass
 
     @staticmethod
-    def remove_networks(names: List[str]) -> None:
+    def remove_networks(names: List[str], logger: logging.Logger) -> bool:
         """
         Removes a network
 
         :param name: the name of the network to remove
-        :return: None
+        :param logger: the logger to use for logging
+        :return: True if at least one of the networks were successfully removed
         """
         client_1 = docker.from_env()
         networks = client_1.networks.list()
+        network_removed = False
         for net in networks:
             if net.name in names:
-                Logger.__call__().get_logger().info(f"Removing network: {net.name}")
+                logger.info(f"Removing network: {net.name}")
                 try:
                     net.remove()
+                    network_removed = True
                 except Exception:
                     pass
+        return network_removed
 
     @staticmethod
-    def rm_all_networks() -> None:
+    def rm_all_networks(logger: logging.Logger) -> None:
         """
         A utility function for removing all csle networks
 
+        :param logger: the logger to use for logging
         :return: None
         """
         client_1 = docker.from_env()
         networks = client_1.networks.list()
         networks = list(filter(lambda x: constants.CSLE.NAME in x.name, networks))
         for net in networks:
-            Logger.__call__().get_logger().info(f"Removing network:{net.name}")
-            ContainerController.remove_network(name=net.name)
+            logger.info(f"Removing network:{net.name}")
+            ContainerController.remove_network(name=net.name, logger=logger)
 
     @staticmethod
-    def rm_network(name) -> bool:
+    def rm_network(name, logger: logging.Logger) -> bool:
         """
         A utility function for removing a network with a specific name
 
+        :param name: the name of the network to remove
+        :param logger: the logger to use for logging
         :return: True if it was removed or False otherwise
         """
         client_1 = docker.from_env()
@@ -654,7 +672,7 @@ class ContainerController:
         networks = list(filter(lambda x: constants.CSLE.NAME in x.name, networks))
         for net in networks:
             if net == name:
-                ContainerController.remove_network(name=net.name)
+                ContainerController.remove_network(name=net.name, logger=logger)
                 return True
         return False
 
@@ -666,7 +684,6 @@ class ContainerController:
         :param cmd: the command to run
         :return: None
         """
-
         if cmd == constants.MANAGEMENT.LIST_STOPPED:
             names = ContainerController.list_all_stopped_containers()
             Logger.__call__().get_logger().info(names)
@@ -688,7 +705,7 @@ class ContainerController:
             networks = ContainerController.list_all_networks()
             Logger.__call__().get_logger().info(networks)
         elif cmd == constants.MANAGEMENT.RM_NETWORKS:
-            ContainerController.rm_all_networks()
+            ContainerController.rm_all_networks(logger=Logger.__call__().get_logger())
         else:
             raise ValueError("Command: {} not recognized".format(cmd))
 
@@ -700,8 +717,7 @@ class ContainerController:
         :param emulation_env_config: the emulation env config
         :return: the list of IP addresses
         """
-        hostname = socket.gethostname()
-        return [socket.gethostbyname(hostname)]
+        return [GeneralUtil.get_host_ip()]
 
     @staticmethod
     def get_docker_stats_managers_ports(emulation_env_config: EmulationEnvConfig) -> List[int]:
@@ -714,13 +730,16 @@ class ContainerController:
         return [emulation_env_config.docker_stats_manager_config.docker_stats_manager_port]
 
     @staticmethod
-    def get_docker_stats_managers_info(emulation_env_config: EmulationEnvConfig, active_ips: List[str]) \
+    def get_docker_stats_managers_info(emulation_env_config: EmulationEnvConfig, active_ips: List[str],
+                                       physical_host_ip: str, logger: logging.Logger) \
             -> DockerStatsManagersInfo:
         """
         Extracts the information of the Docker stats managers for a given emulation
 
         :param emulation_env_config: the configuration of the emulation
         :param active_ips: list of active IPs
+        :param physical_host_ip: the ip of the physical host
+        :param logger: the logger to use for logging
         :return: a DTO with the status of the Docker stats managers
         """
         docker_stats_managers_ips = ContainerController.get_docker_stats_managers_ips(
@@ -730,7 +749,7 @@ class ContainerController:
         docker_stats_managers_statuses = []
         docker_stats_managers_running = []
         for ip in docker_stats_managers_ips:
-            if ip not in active_ips:
+            if ip not in active_ips or ip != physical_host_ip:
                 continue
             running = False
             status = None
@@ -739,7 +758,7 @@ class ContainerController:
                     port=emulation_env_config.docker_stats_manager_config.docker_stats_manager_port, ip=ip)
                 running = True
             except Exception as e:
-                Logger.__call__().get_logger().debug(
+                logger.debug(
                     f"Could not fetch Docker stats manager status on IP:{ip}, error: {str(e)}, {repr(e)}")
             if status is not None:
                 docker_stats_managers_statuses.append(status)
