@@ -1,3 +1,4 @@
+import logging
 from typing import List
 import grpc
 import time
@@ -18,16 +19,17 @@ class SDNControllerManager:
     """
 
     @staticmethod
-    def start_ryu_manager(emulation_env_config: EmulationEnvConfig) -> None:
+    def start_ryu_manager(emulation_env_config: EmulationEnvConfig, logger: logging.Logger) -> None:
         """
         Utility method for starting the Ryu manager
 
         :param emulation_env_config: the emulation env config
+        :param logger: the logger to use for logging
         :return: None
         """
         # Connect
         EmulationUtil.connect_admin(emulation_env_config=emulation_env_config,
-                                    ip=emulation_env_config.sdn_controller_config.container.get_ips()[0],
+                                    ip=emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip,
                                     create_producer=False)
 
         # Check if ryu_manager is already running
@@ -36,12 +38,12 @@ class SDNControllerManager:
         o, e, _ = EmulationUtil.execute_ssh_cmd(
             cmd=cmd,
             conn=emulation_env_config.get_connection(
-                ip=emulation_env_config.sdn_controller_config.container.get_ips()[0]))
+                ip=emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip))
 
         if constants.COMMANDS.SEARCH_RYU_MANAGER not in str(o):
-            Logger.__call__().get_logger().info(
+            logger.info(
                 f"Starting ryu manager on node: "
-                f"{emulation_env_config.sdn_controller_config.container.get_ips()[0]}")
+                f"{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}")
 
             # Stop old background job if running
             cmd = (constants.COMMANDS.SUDO + constants.COMMANDS.SPACE_DELIM + constants.COMMANDS.PKILL +
@@ -49,7 +51,7 @@ class SDNControllerManager:
             o, e, _ = EmulationUtil.execute_ssh_cmd(
                 cmd=cmd,
                 conn=emulation_env_config.get_connection(
-                    ip=emulation_env_config.sdn_controller_config.container.get_ips()[0]))
+                    ip=emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip))
 
             # Start the ryu_manager
             cmd = constants.COMMANDS.START_RYU_MANAGER.format(
@@ -60,7 +62,7 @@ class SDNControllerManager:
             o, e, _ = EmulationUtil.execute_ssh_cmd(
                 cmd=cmd,
                 conn=emulation_env_config.get_connection(
-                    ip=emulation_env_config.sdn_controller_config.container.get_ips()[0]))
+                    ip=emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip))
             time.sleep(2)
 
     @staticmethod
@@ -73,11 +75,12 @@ class SDNControllerManager:
         """
         # Connect
         EmulationUtil.connect_admin(emulation_env_config=emulation_env_config,
-                                    ip=emulation_env_config.sdn_controller_config.container.get_ips()[0],
+                                    ip=emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip,
                                     create_producer=False)
 
-        Logger.__call__().get_logger().info(f"Stopping ryu manager on node: "
-                                            f"{emulation_env_config.sdn_controller_config.container.get_ips()[0]}")
+        Logger.__call__().get_logger().info(
+            f"Stopping ryu manager on node: "
+            f"{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}")
 
         # Stop background job
         cmd = (constants.COMMANDS.SUDO + constants.COMMANDS.SPACE_DELIM + constants.COMMANDS.PKILL +
@@ -85,22 +88,23 @@ class SDNControllerManager:
         o, e, _ = EmulationUtil.execute_ssh_cmd(
             cmd=cmd,
             conn=emulation_env_config.get_connection(
-                ip=emulation_env_config.sdn_controller_config.container.get_ips()[0]))
+                ip=emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip))
 
         time.sleep(2)
 
     @staticmethod
-    def get_ryu_status(emulation_env_config: EmulationEnvConfig) -> \
+    def get_ryu_status(emulation_env_config: EmulationEnvConfig, logger: logging.Logger) -> \
             csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
         """
         Method for querying the RyuManager about the status of the Ryu SDN controller
 
         :param emulation_env_config: the emulation config
+        :param logger: the logger to use for logging
         :return: a RyuDTO with the status
         """
-        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config)
+        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config, logger=logger)
         ryu_dto = SDNControllerManager.get_ryu_status_by_port_and_ip(
-            ip=emulation_env_config.sdn_controller_config.container.get_ips()[0],
+            ip=emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip,
             port=emulation_env_config.sdn_controller_config.manager_port)
         return ryu_dto
 
@@ -123,45 +127,51 @@ class SDNControllerManager:
             return ryu_dto
 
     @staticmethod
-    def stop_ryu(emulation_env_config: EmulationEnvConfig) -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
+    def stop_ryu(emulation_env_config: EmulationEnvConfig, logger: logging.Logger) \
+            -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
         """
         Method for requesting the RyuManager to stop the RYU SDN controller
 
         :param emulation_env_config: the emulation env config
         :return: a RyuDTO with the status of the server
         """
-        Logger.__call__().get_logger().info(
+        logger.info(
             f"Stopping RYU SDN controller on container: "
-            f"{emulation_env_config.sdn_controller_config.container.get_ips()[0]}")
-        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config)
+            f"{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}")
+        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config, logger=logger)
 
         # Open a gRPC session
         with grpc.insecure_channel(
-                f'{emulation_env_config.sdn_controller_config.container.get_ips()[0]}:'
+                f'{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}:'
                 f'{emulation_env_config.sdn_controller_config.manager_port}') as channel:
             stub = csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub(channel)
             ryu_dto = csle_collector.ryu_manager.query_ryu_server.stop_ryu(stub)
             return ryu_dto
 
     @staticmethod
-    def start_ryu(emulation_env_config: EmulationEnvConfig) -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
+    def start_ryu(emulation_env_config: EmulationEnvConfig, physical_server_ip: str, logger: logging.Logger) \
+            -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
         """
         Method for requesting the RyuManager to start the Ryu SDN controller
 
         :param emulation_env_config: the emulation env config
+        :param physical_server_ip: the ip of the physical server
+        :param logger: the logger to use for logging
         :return: an RyuDTO with the status of the server
         """
-        Logger.__call__().get_logger().info(
+        if emulation_env_config.sdn_controller_config.container.physical_host_ip != physical_server_ip:
+            return
+        logger.info(
             f"Starting Ryu SDN controller on container: "
-            f"{emulation_env_config.sdn_controller_config.container.get_ips()[0]}")
-        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config)
+            f"{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}")
+        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config, logger=logger)
 
         # Open a gRPC session
         with grpc.insecure_channel(
-                f'{emulation_env_config.sdn_controller_config.container.get_ips()[0]}:'
+                f'{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}:'
                 f'{emulation_env_config.sdn_controller_config.manager_port}') as channel:
             stub = csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub(channel)
-            Logger.__call__().get_logger().info(
+            logger.info(
                 f"Starting RYU, port: {emulation_env_config.sdn_controller_config.controller_port}, "
                 f"web_port: {emulation_env_config.sdn_controller_config.controller_web_api_port}, "
                 f"controller: {emulation_env_config.sdn_controller_config.controller_module_name}")
@@ -172,47 +182,52 @@ class SDNControllerManager:
             return ryu_dto
 
     @staticmethod
-    def start_ryu_monitor(emulation_env_config: EmulationEnvConfig) \
+    def start_ryu_monitor(emulation_env_config: EmulationEnvConfig, physical_server_ip: str, logger: logging.Logger) \
             -> csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
         """
         Method for requesting the RyuManager to start the Ryu monitor
 
         :param emulation_env_config: the emulation env config
+        :param physical_server_ip: ip of the physical server
+        :param logger: the logger to use for logging
         :return: an RyuDTO with the status
         """
-        Logger.__call__().get_logger().info(
+        if emulation_env_config.sdn_controller_config.container.physical_host_ip != physical_server_ip:
+            return
+        logger.info(
             f"Starting the ryu monitor on container: "
-            f"{emulation_env_config.sdn_controller_config.container.get_ips()[0]}")
-        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config)
+            f"{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}")
+        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config, logger=logger)
 
         # Open a gRPC session
         with grpc.insecure_channel(
-                f'{emulation_env_config.sdn_controller_config.container.get_ips()[0]}:'
+                f'{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}:'
                 f'{emulation_env_config.sdn_controller_config.manager_port}') as channel:
             stub = csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub(channel)
             ryu_dto = csle_collector.ryu_manager.query_ryu_server.start_ryu_monitor(
-                stub, kafka_ip=emulation_env_config.kafka_config.container.get_ips()[0],
+                stub, kafka_ip=emulation_env_config.kafka_config.container.docker_gw_bridge_ip,
                 kafka_port=emulation_env_config.kafka_config.kafka_port,
                 time_step_len=emulation_env_config.sdn_controller_config.time_step_len_seconds)
             return ryu_dto
 
     @staticmethod
-    def stop_ryu_monitor(emulation_env_config: EmulationEnvConfig) -> \
+    def stop_ryu_monitor(emulation_env_config: EmulationEnvConfig, logger: logging.Logger) -> \
             csle_collector.ryu_manager.ryu_manager_pb2.RyuDTO:
         """
         Method for requesting the RyuManager to stop the ryu monitor
 
         :param emulation_env_config: the emulation env config
+        :param logger: the logger to use for logging
         :return: an RyuDTO with the status of the server
         """
-        Logger.__call__().get_logger().info(
+        logger.info(
             f"Stopping Ryu monitor on container: "
-            f"{emulation_env_config.sdn_controller_config.container.get_ips()[0]}")
-        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config)
+            f"{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}")
+        SDNControllerManager.start_ryu_manager(emulation_env_config=emulation_env_config, logger=logger)
 
         # Open a gRPC session
         with grpc.insecure_channel(
-                f'{emulation_env_config.sdn_controller_config.container.get_ips()[0]}:'
+                f'{emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip}:'
                 f'{emulation_env_config.sdn_controller_config.manager_port}') as channel:
             stub = csle_collector.ryu_manager.ryu_manager_pb2_grpc.RyuManagerStub(channel)
             ryu_dto = csle_collector.ryu_manager.query_ryu_server.stop_ryu_monitor(stub)
@@ -226,7 +241,7 @@ class SDNControllerManager:
         :param emulation_env_config: the emulation env config
         :return: the list of IP addresses
         """
-        return [emulation_env_config.sdn_controller_config.container.get_ips()[0]]
+        return [emulation_env_config.sdn_controller_config.container.docker_gw_bridge_ip]
 
     @staticmethod
     def get_ryu_managers_ports(emulation_env_config: EmulationEnvConfig) -> List[int]:
@@ -239,12 +254,15 @@ class SDNControllerManager:
         return [emulation_env_config.sdn_controller_config.manager_port]
 
     @staticmethod
-    def get_ryu_managers_info(emulation_env_config: EmulationEnvConfig, active_ips: List[str]) -> RyuManagersInfo:
+    def get_ryu_managers_info(emulation_env_config: EmulationEnvConfig, active_ips: List[str],
+                              logger: logging.Logger, physical_server_ip: str) -> RyuManagersInfo:
         """
         Extracts the information of the Ryu managers for a given emulation
 
         :param emulation_env_config: the configuration of the emulation
         :param active_ips: list of active IPs
+        :param logger: the logger to use for logging
+        :param physical_server_ip: the ip of the physical server
         :return: a DTO with the status of the Ryu managers
         """
         ryu_managers_ips = SDNControllerManager.get_ryu_managers_ips(emulation_env_config=emulation_env_config)
@@ -252,7 +270,8 @@ class SDNControllerManager:
         ryu_managers_statuses = []
         ryu_managers_running = []
         for ip in ryu_managers_ips:
-            if ip not in active_ips:
+            if ip not in active_ips or not EmulationUtil.physical_ip_match(
+                    emulation_env_config=emulation_env_config, ip=ip, physical_host_ip=physical_server_ip):
                 continue
             status = None
             try:
@@ -261,8 +280,7 @@ class SDNControllerManager:
                 running = True
             except Exception as e:
                 running = False
-                Logger.__call__().get_logger().debug(
-                    f"Could not fetch Ryu manager status on IP:{ip}, error: {str(e)}, {repr(e)}")
+                logger.debug(f"Could not fetch Ryu manager status on IP:{ip}, error: {str(e)}, {repr(e)}")
             if status is not None:
                 ryu_managers_statuses.append(status)
             else:
