@@ -22,6 +22,7 @@ from csle_common.dao.emulation_config.ossec_ids_manager_config import OSSECIDSMa
 from csle_common.dao.emulation_config.docker_stats_manager_config import DockerStatsManagerConfig
 from csle_common.dao.emulation_config.beats_config import BeatsConfig
 from csle_common.dao.emulation_config.elk_config import ElkConfig
+from csle_common.dao.emulation_config.container_network import ContainerNetwork
 from csle_common.util.ssh_util import SSHUtil
 from csle_common.logging.log import Logger
 import csle_collector.constants.constants as collector_constants
@@ -370,29 +371,46 @@ class EmulationEnvConfig:
         with io.open(json_file_path, 'w', encoding='utf-8') as f:
             f.write(json_str)
 
+    @staticmethod
+    def from_json_file(json_file_path: str) -> "EmulationEnvConfig":
+        """
+        Reads a json file and converts it to a DTO
+
+        :param json_file_path: the json file path
+        :return: the converted DTO
+        """
+        import io
+        import json
+        with io.open(json_file_path, 'r') as f:
+            json_str = f.read()
+        return EmulationEnvConfig.from_dict(json.loads(json_str))
+
     def copy(self) -> "EmulationEnvConfig":
         """
         :return: a copy of the DTO
         """
         return EmulationEnvConfig.from_dict(self.to_dict())
 
-    def create_execution_config(self, ip_first_octet: int) -> "EmulationEnvConfig":
+    def create_execution_config(self, ip_first_octet: int, physical_servers: List[str]) -> "EmulationEnvConfig":
         """
         Creates an execution config from the base config
 
         :param ip_first_octet:  the id of the execution
+        :param physical_servers: the list of physical servers
         :return: the created execution config
         """
         config = self.copy()
         config.execution_id = ip_first_octet
-        config.containers_config = config.containers_config.create_execution_config(ip_first_octet=ip_first_octet)
+        config.containers_config = config.containers_config.create_execution_config(
+            ip_first_octet=ip_first_octet, physical_servers=physical_servers)
         config.users_config = config.users_config.create_execution_config(ip_first_octet=ip_first_octet)
         config.flags_config = config.flags_config.create_execution_config(ip_first_octet=ip_first_octet)
         config.vuln_config = config.vuln_config.create_execution_config(ip_first_octet=ip_first_octet)
         config.topology_config = config.topology_config.create_execution_config(ip_first_octet=ip_first_octet)
         config.traffic_config = config.traffic_config.create_execution_config(ip_first_octet=ip_first_octet)
         config.resources_config = config.resources_config.create_execution_config(ip_first_octet=ip_first_octet)
-        config.kafka_config = config.kafka_config.create_execution_config(ip_first_octet=ip_first_octet)
+        config.kafka_config = config.kafka_config.create_execution_config(ip_first_octet=ip_first_octet,
+                                                                          physical_servers=physical_servers)
         config.services_config = config.services_config.create_execution_config(ip_first_octet=ip_first_octet)
         config.ovs_config = config.ovs_config.create_execution_config(ip_first_octet=ip_first_octet)
         config.host_manager_config = config.host_manager_config.create_execution_config(ip_first_octet=ip_first_octet)
@@ -403,14 +421,25 @@ class EmulationEnvConfig:
         config.docker_stats_manager_config = config.docker_stats_manager_config.create_execution_config(
             ip_first_octet=ip_first_octet)
         config.elk_config = config.elk_config.create_execution_config(
-            ip_first_octet=ip_first_octet)
+            ip_first_octet=ip_first_octet, physical_servers=physical_servers)
         config.beats_config = config.beats_config.create_execution_config(ip_first_octet=ip_first_octet)
         if config.sdn_controller_config is not None:
             config.sdn_controller_config = config.sdn_controller_config.create_execution_config(
-                ip_first_octet=ip_first_octet)
+                ip_first_octet=ip_first_octet, physical_servers=physical_servers)
         static_attacker_sequences = {}
         for k, v in config.static_attacker_sequences.items():
             static_attacker_sequences[k] = list(map(lambda x: x.create_execution_config(ip_first_octet=ip_first_octet),
                                                     config.static_attacker_sequences[k]))
         config.static_attacker_sequences = static_attacker_sequences
         return config
+
+    def get_network_by_name(self, net_name: str) -> ContainerNetwork:
+        """
+        Gets the network by name from the config
+
+        :param net_name: the name of the network
+        :return: the network with the given name
+        """
+        for c in self.containers_config.networks:
+            if c.name == net_name:
+                return c
